'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.matchImageSnapshotOptions = matchImageSnapshotOptions;
exports.matchImageSnapshotResults = matchImageSnapshotResults;
exports.matchImageSnapshotPlugin = matchImageSnapshotPlugin;
exports.addMatchImageSnapshotPlugin = addMatchImageSnapshotPlugin;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _diffSnapshot = require('jest-image-snapshot/src/diff-snapshot');

var _lodash = require('lodash.kebabcase');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; } /**
                                                                                                                                                                                                                              * Copyright (c) 2018-present The Palmer Group
                                                                                                                                                                                                                              *
                                                                                                                                                                                                                              * This source code is licensed under the MIT license found in the
                                                                                                                                                                                                                              * LICENSE file in the root directory of this source tree.
                                                                                                                                                                                                                              */

let snapshotOptions = {};
let snapshotResults = {};
let snapshotRunning = false;

function matchImageSnapshotOptions(options = {}) {
  snapshotOptions = options;
  snapshotRunning = true;
  return null;
}

function matchImageSnapshotResults() {
  snapshotRunning = false;
  return snapshotResults;
}

function matchImageSnapshotPlugin({ path: screenshotPath }) {
  if (!snapshotRunning) {
    return null;
  }

  const {
    updateSnapshots,
    options: {
      failureThreshold = 0,
      failureThresholdType = 'pixel'
    } = {}
  } = snapshotOptions,
        options = _objectWithoutProperties(snapshotOptions.options, ['failureThreshold', 'failureThresholdType']);

  const receivedImageBuffer = _fs2.default.readFileSync(screenshotPath);
  const screenshotFileName = screenshotPath.slice(screenshotPath.lastIndexOf('/'));
  const screenshotDir = screenshotPath.replace(screenshotFileName, '');
  const snapshotIdentifier = (0, _lodash2.default)(screenshotFileName.replace('.png', ''));
  const snapshotsDir = screenshotDir.replace('screenshots', 'snapshots');
  const snapshotPath = `${snapshotsDir}/${snapshotIdentifier}-snap.png`;

  snapshotResults = (0, _diffSnapshot.diffImageToSnapshot)(_extends({
    snapshotsDir,
    receivedImageBuffer,
    snapshotIdentifier,
    failureThreshold,
    failureThresholdType,
    updateSnapshot: updateSnapshots
  }, options));

  const { pass, added, updated, diffOutputPath } = snapshotResults;

  if (!pass && !added && !updated) {
    return {
      path: diffOutputPath
    };
  }

  return {
    path: snapshotPath
  };
}

function addMatchImageSnapshotPlugin(on) {
  on('task', {
    'Matching image snapshot': matchImageSnapshotOptions,
    'Recording snapshot results': matchImageSnapshotResults
  });
  on('after:screenshot', matchImageSnapshotPlugin);
}